#!/usr/bin/perl
#
#  Voicemail transcription
#
#  This script sends all voicemail messages off for transcription, if
#  customer has the feature enabled.
#

use strict;
use Sys::Hostname;
use File::Copy;
use Digest::MD5 qw/md5_hex/;

# parameters Asterisk sends us
my ($call, $VM_SPOOL_DIR, $context, $mailbox, $fromfolder, $msgnum) = @ARGV;

# This is what Asterisk would normally use, for the mailcmd (voicemail.conf)
# Must be a command that accepts an email via stdin (pipe)
# This is only used for fallback, in case the CP can't be contacted.
my $asterisk_default_mailcmd = '/usr/sbin/sendmail -t';

my $settings = get_settings($context, $mailbox);
if (!defined($settings))
{
	error("A problem occured while getting settings");
	exit;
}

# for Connect, check if the guest has vmx enabled,
# if it doesn't call sendmail and return
if ($settings->{unbound})
{
	if (!$settings->{vmx}) {
		my $ret = sendmail();
		if (!$ret)
		{
			error("A problem occured while sending the message with $asterisk_default_mailcmd");
		}
		exit;
	}
}

# send the email via cp
my $rand_str = get_rand_str();

# Make temporary copies of txt and wav files which
# can be reordered at any time by Asterisk.
# This is to avoid race conditions.
# In case "Delete When Emailed" option is enabled,
# move the files, rather than copy.
my $msg_txt = `mktemp -p /tmp "${rand_str}.txt"`;
chomp $msg_txt;
my $from = sprintf("${VM_SPOOL_DIR}$context/$mailbox/$fromfolder/msg%04d.txt", $msgnum);
copy_files($from, $msg_txt, $settings->{vmx_delete});

my $msg_wav = `mktemp -p /tmp "${rand_str}.wav"`;
chomp $msg_wav;
$from = sprintf("${VM_SPOOL_DIR}$context/$mailbox/$fromfolder/msg%04d.wav", $msgnum);
copy_files($from, $msg_wav, $settings->{vmx_delete});

# trixbox or PBXtra/Connect?
my $vpn_file = '/etc/sysconfig/vpn';
if (!open(VPN, "<$vpn_file"))
{
	error("Could not open $vpn_file for reading: $!");
	exit;
}
my $stype = 'pbxtra';
while (<VPN>)
{
	if (/^GW.*trixbox/)
	{
		$stype = 'trixbox';
		last;
	}
}
close(VPN);

my $cgi_host = 'cp.' . ($stype eq 'trixbox' ? 'trixbox' : 'fonality') . '.com';
my $url = "https://$cgi_host/vm_notify.cgi";
my $params = "call=$call&sid=" . $settings->{server_id} . "&mb=$mailbox&rand_str=$rand_str&user=" .
	$settings->{user} . "&hash=" . $settings->{hash};

my $ret = do_wget($url, $params);
if (!$ret)
{
	error("Couldn't contact cp, sending with $asterisk_default_mailcmd");
	$ret = sendmail();
	if ($ret)
	{
		unlink $msg_txt, $msg_wav;
	} else
	{
		error("A problem occured while sending the message with $asterisk_default_mailcmd");
	}
}

sub copy_files
{
	my $from = shift;
	my $to = shift;
	my $move = shift;

	if ($move) {
		if (!move($from, $to)) {
			error("Move failed $from -> $to: $!");
		}
	} else {
		if (!copy($from, $to)) {
			error("Copy failed $from -> $to: $!");
		}
	}
}

# Use wget rather than Perl here.  Premise boxes don't have the correct modules
# for dealing with SSL in Perl, and we don't want to risk any redirection to SSL causing issues.
sub do_wget
{
	my ($url) = shift;
	my ($params) = shift;
	
	# file for temporary wget output
	my $tmpfile = `/bin/mktemp`;
	chomp $tmpfile;
	
	my $timeout_secs = 60 * 2;
	my $wget = '/usr/bin/wget';
	my $args = "-t 1 -T $timeout_secs -O $tmpfile --post-data='$params' '$url'";
	my $cmd = "$wget $args >/dev/null 2>&1";
	
	if (system($cmd) != 0)
	{
		error("Could not fetch URL: $url");
		return undef;
	}
	
	if (!open INP, "<$tmpfile")
	{
		error("Could not open $tmpfile for reading: $!");
		return undef;
	}
	
	my @lines = <INP>;
	close INP;
	unlink $tmpfile;
	
	return join('', @lines);
}

sub get_rand_str
{
	my @chars = ('a'..'z', 'A'..'Z', '0'..'9');
	my $str;
	foreach (1..16)
	{
		$str .= $chars[int(rand @chars)];
	}
	return $str;
}

sub sendmail
{
	if (!open(OUT, "| $asterisk_default_mailcmd"))
	{
		error("$asterisk_default_mailcmd failed: $!");
		return undef;
	}

	undef $/;
	return print OUT <STDIN>;
}

sub get_settings
{
	my $context = shift;
	my $mailbox = shift;
	my ($server_id) = hostname;
	$server_id =~ s/(?:trixbox|pbxtra)//;
	my $filename = '/etc/asterisk/fonality/globals.conf';
	my $vm_default = '/etc/asterisk/vm_default.conf';
	my $ret = {};
	
	if (-e '/etc/asterisk/unbound')
	{
		$ret->{unbound} = 1;
		($server_id) = $context =~ /-(\d+)$/;
		$filename = "/data/$server_id/etc/asterisk/fonality/globals.conf";
		$vm_default = "/data/$server_id/etc/asterisk/vm_default.conf";
	}
	
	$ret->{server_id} = $server_id;

	# read globals.conf
	if (!open(GLOBALS, "<$filename"))
	{
		error("Could not open $filename for reading: $!");
		return undef;
	}
	while (<GLOBALS>)
	{
		if (!exists $ret->{vmx} && $ret->{unbound} && /^VMX=1/)
		{
			$ret->{vmx} = 1;
		}
		if ((!exists $ret->{user} || !exists $ret->{hash}) && /^IAXINFO\d?=(s\d{4,6}):([^;\s]+)/)
		{
			$ret->{user} = $1;
			$ret->{hash} = md5_hex($2);
		}
		if (exists $ret->{vmx}, exists $ret->{user}, exists $ret->{hash})
		{
			last;
		}
	}
	close(GLOBALS);
	
	# read vm_default.conf
	if (!open(VM_DEFAULT, "<$vm_default"))
	{
		error("Could not open $vm_default for reading: $!");
		return undef;
	}
	while (<VM_DEFAULT>)
	{
		if (/^$mailbox/)
		{
			if (/vmx_delete=1/)
			{
				$ret->{vmx_delete} = 1;
			}
			last;
		}
	}
	close(VM_DEFAULT);
	
	return $ret;
}

sub error
{
	my ($msg) = @_;

	my $ts = `date +'%x %T'`;
	chomp $ts;
	open OUT, ">>/tmp/vm_transcribe.errorlog";
	print OUT "[$ts] $msg\n";
	close OUT;
}

